/*
Copyright (C) 2006 Parallel Realities

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include "laserGrid.h"

void directLaserBeam(float x, float y, LaserGrid *laserGrid, Texture *beam, int color, int direction)
{
	int dx = 0;
	int dy = 0;
	
	x += 0.5;
	y += 0.5;
	
	switch (direction)
	{
		case 0:
			dy = -1;
			break;
			
		case 1:
			dx = 1;
			break;
			
		case 2:
			dy = 1;
			break;
			
		case 3:
			dx = -1;
			break;
	}
	
	laserGrid->grid[(int)x][(int)y][0] = color;
	laserGrid->grid[(int)x][(int)y][1] = direction;
	
	bool hit = false;
	
	int center = LaserGrid::GRID_SPACE;
	
	float startX, startY, endX, endY;
	
	startX = (64 + (x * LaserGrid::GRID_SPACE));
	startY = (24 + (y * LaserGrid::GRID_SPACE));
	
	endX = (startX + center);
	endY = (startY + center);
	
	while (!hit)
	{
		glColor4f(1.0, 0.0, 0.0, 0.5);
		
		while (startX != endX && startY != endY)
		{
			graphics->blit(beam, (int)startX, (int)endY, true);
			startX += dx;
			startY += dy;
		}
		
		x += dx;
		y += dy;
		
		startX = 64 + (x * LaserGrid::GRID_SPACE);
		startY = 32 + (y * LaserGrid::GRID_SPACE);
	
		endX = (startX + center);
		endY = (startY + center);
		
		hit = true;
		
		if ((x >= LaserGrid::GRID_X) || (y >= LaserGrid::GRID_Y))
		{
			hit = true;
		}
	}
}

bool doLaserGridPuzzle(int seed)
{
	graphics->clearScreen();
	graphics->delay(500, false);
	
	Texture *beam = getRequiredTexture("textures/game/laserBeam.png");
	
	//int puzzleState = PUZZLE_STATE_UNFINISHED;
	
	float timer = 250;
	
	GLColor darkGrey = GLColor::grey;
	darkGrey.color[0] = darkGrey.color[1] = darkGrey.color[2] = 0.05;
	
	LaserGrid laserGrid;
	
	for (int x = 0 ; x < LaserGrid::GRID_X ; x++)
	{
		for (int y = 0 ; y < LaserGrid::GRID_Y ; y++)
		{
			laserGrid.grid[x][y][0] = laserGrid.grid[x][y][1] = 0;
		}
	}
	
	while (timer > 0)
	{
		engine->doTimeDifference();
		
		graphics->updateScreen();
		engine->getInput();
		
		graphics->clearScreen();
		
		graphics->setMode(MODE_2D);
		
		int gx1 = 64;
		int gx2 = gx1 + (LaserGrid::GRID_SPACE * LaserGrid::GRID_X);
		
		int gy1 = 32;
		int gy2 = gy1 + (LaserGrid::GRID_SPACE * LaserGrid::GRID_Y);
		
		for (int x = gx1 ; x < gx2 ; x += LaserGrid::GRID_SPACE)
		{
			graphics->drawLine(x, gy1, x, gy2, darkGrey);
		}
		
		graphics->drawLine(gx2, gy1, gx2, gy2, darkGrey);
		
		for (int y = gy1 ; y < gy2 ; y += LaserGrid::GRID_SPACE)
		{
			graphics->drawLine(gx1, y, gx2, y, darkGrey);
		}
		
		graphics->drawLine(gx1, gy2, gx2, gy2, darkGrey);
		
		directLaserBeam(1, 0, &laserGrid, beam, 0, 0); 
	}
	
	return true;
}
